def link(request, path):
  """return a permalink"""
  base_url = request.application_url.rstrip('/')
  if not path:
    return base_url + '/'
  if not isinstance(path, basestring):
    path = '/'.join(path)
  return '%s/%s' % (base_url, path.lstrip('/'))
  



"""
decoupage: a dynamic file server
"""

# TODO:

# handle files with `#`s like like `#index.ini`
# -> http://k0s.org/portfolio/ideas/#index.ini#
#
# oops. Handle it better
# - either # is a magic hide character
# - or you urlescape that guy


import os
import sys

from collections import OrderedDict
from contenttransformer.app import FileTypeTransformer
from contenttransformer.app import transformers
from datetime import datetime
from formatters import formatters
from genshi.builder import Markup
from genshi.template import TemplateLoader
from genshi.template.base import TemplateError
from genshi.template.base import TemplateSyntaxError
from martini.config import ConfigMunger
from paste.fileapp import FileApp
from pkg_resources import iter_entry_points
from pkg_resources import load_entry_point
from pkg_resources import resource_filename
from webob import Request, Response, exc

transformers = transformers()
string = (str, unicode)


class Decoupage(object):

    ### class level variables
    defaults = { 'auto_reload': False,
                 'configuration': None,
                 'directory': None, # directory to serve
                 'cascade': True, # whether to cascade configuration
                 'template': 'index.html', # XXX see below
                 'template_directories': '', # list of directories to look for templates
                 'charset': 'utf-8', # content encoding for index.html files; -> `Content-Type: text/html; charset=ISO-8859-1`
                 }

    def __init__(self, **kw):

        # set defaults from app configuration
        for key, default_value in self.defaults.items():

            value = kw.get(key, default_value)

            # handle non-string bools
            if isinstance(default_value, bool) and isinstance(value, string):
                value = {'true': True,
                         'false': False}[value.lower()]
                # TODO: error handling for bad strings

            setattr(self, key, value)


        # configure defaults
        assert self.directory, "Decoupage: directory not specified"
        self.directory = self.directory.rstrip(os.path.sep)
        assert os.path.isdir(self.directory), "'%s' is not a directory" % self.directory
        self.template_directories = self.template_directories.split() # no spaces in directory names, for now

        for directory in self.template_directories:
            assert os.path.isdir(directory), "Decoupage template directory %s does not exist!" % directory

        # static file server
        self.fileserver = FileApp

        # pluggable formats
        s = 'format.'
        _format_args = [ (i.split(s, 1)[-1], j) for i, j in kw.items()
                         if i.startswith(s) ]
        format_args = {}
        for i, j in _format_args:
            assert i.count('.') == 1, 'Illegal string or something'
            format_name, var_name = i.split('.')
            format_args.setdefault(format_name, {})[var_name] = j
        self.formats = {}
        for _format in iter_entry_points('decoupage.formats'):
            try:
                _cls = _format.load()
                _instance = _cls(self, **format_args.get(_format.name, {}))
            except Exception, e:
                # record the error, but persist
                print >> sys.stderr, "Couldn't load format: %s" % _format
                print >> sys.stderr, e
                continue
            self.formats[_format.name] = _instance

        # pluggable index data formatters
        self.formatters = {}
        for formatter in iter_entry_points('decoupage.formatters'):
            try:
                _formatter = formatter.load()
                template_dir = resource_filename(formatter.module_name, 'templates')
                if template_dir not in self.template_directories and os.path.isdir(template_dir):
                    self.template_directories.append(template_dir)
            except Exception, e:
                # record the error, but persist
                print >> sys.stderr, "Couldn't load formatter: %s" % formatter
                print >> sys.stderr, e
                continue
            self.formatters[formatter.name] = _formatter

        # template loader
        self.loader = TemplateLoader(self.template_directories,
                                     variable_lookup="lenient",
                                     auto_reload=self.auto_reload)


    ### methods dealing with HTTP

    def __call__(self, environ, start_response):
        """WSGI application"""

        # boilerplate: request and filename
        request = Request(environ)
        filename = request.path_info.strip('/')
        path = os.path.join(self.directory, filename)

        # check to see what we have to serve
        if os.path.exists(path):

            if os.path.isdir(path):
                # serve an index
                if request.path_info.endswith('/'):
                    res = self.get(request)
                else:
                    res = exc.HTTPMovedPermanently(add_slash=True)
                return res(environ, start_response)

            else:
                # serve a file
                conf = self.conf(request.path_info.rsplit('/',1)[0])
                if '/transformer' in conf:
                    args = [i.split('=', 1) for i in conf['/transformer'].split(',') if '=' in i]
                    kwargs = {}
                    for i in conf:
                        if i.startswith('/'):
                            name = i[1:]
                            if name in transformers:
                                kwargs[name] = dict([j.split('=', 1) for j in conf[i].split(',') if '=' in j])
                    fileserver = FileTypeTransformer(*args, **kwargs)
                else:
                    fileserver = self.fileserver

                fileserver = fileserver(path)
                return fileserver(environ, start_response)
        else:
            # file does not exist
            conf = self.conf('/')
            data = dict(request=request,
                        title="Not Found")
            template = self.loader.load('HTTPNotFound.html')
            body = template.generate(**data).render('html', doctype='html')
            response = Response(content_type='text/html', body=body, status=404)
            return response(environ, start_response)


    def get(self, request):
        """
        return response to a GET requst
        """

        # ensure a sane path
        path = request.path_info.strip('/')
        directory = os.path.join(self.directory, path)
        path = '/%s' % path

        # get the configuraton
        conf = self.conf(path)

        ### build data dictionary
        # TODO: separate these out into several formatters
        files = self.filedata(path, directory, conf)
        data = {'path': path, 'files': files, 'request': request}

        # add a function to get the path to files
        data['filepath'] = lambda *segments: os.path.join(*([directory] + list(segments)))

        # defaults
        data['directory'] = directory
        data['css'] = ()
        data['scripts'] = ()

        # apply formatters
        formatters = self.get_formatters(path)
        for formatter in formatters:
            formatter(request, data)

        # return an alternate format if specified
        # decoupage.formats should return a 2-tuple:
        # (content_type, body)
        if 'format' in request.GET:
            format_name = request.GET['format']
            if format_name in self.formats:
                _format = self.formats[format_name]
                content_type, body = _format(request, data)
                return Response(content_type=content_type, body=body)

        # render the template
        template = conf.get('/template')
        local_index = False
        if template is None:
            if 'index.html' in [ f['name'] for f in files ]:
                local_index = os.path.join(directory, 'index.html')
                template = local_index
            else:
                template = self.template
        else:
            if not os.path.isabs(template):
                _template = os.path.join(directory, template)
                if os.path.exists(_template):
                    template = _template
                else:
                    for _directory in self.template_directories:
                        if template in os.listdir(_directory):
                            break
                    else:
                        raise IOError("template %s not found" % template)
        try:
            template = self.loader.load(template)
            res = template.generate(**data).render('html', doctype='html')
        except (TemplateError, TemplateSyntaxError), e:
            if local_index:
                print repr(e)
                return self.fileserver(local_index)
            raise

        # set charset if given
        kw = {}
        if self.charset:
            kw['charset'] = self.charset

        # return response
        return Response(content_type='text/html', body=res, **kw)


    ### internal methods

    def filedata(self, path, directory, conf=None):
        conf = conf or OrderedDict()
        files = []

        # get data for files
        filenames = os.listdir(directory)
        for i in filenames:
            filepath = os.path.join(directory, i)
            filetype = 'file'
            if os.path.isdir(filepath):
                filetype = 'directory'
            try:
                modified = os.path.getmtime(filepath)
            except OSError:
                # the file (mysteriously) may not exist by this time(!)
                #  File "/home/jhammel/web/src/decoupage/decoupage/web.py", line 114, in __call__
                #    res = self.get(request)
                #  File "/home/jhammel/web/src/decoupage/decoupage/web.py", line 162, in get
                #    files = self.filedata(path, directory, conf)
                #  File "/home/jhammel/web/src/decoupage/decoupage/web.py", line 246, in filedata
                #    modified = os.path.getmtime(filepath)
                #  File "/home/jhammel/web/lib/python2.6/genericpath.py", line 54, in getmtime
                #    return os.stat(filename).st_mtime
                # OSError: [Errno 2] No such file or directory: '/home/jhammel/web/site/portfolio/ideas/.#index.ini'
                continue # wt{h,f}???

            modified = datetime.fromtimestamp(modified)
            data = {'path' : '%s/%s' % (path.rstrip('/'), i),
                    'name': i,
                    'modified': modified,
                    'type': filetype}
            if filetype == 'file':
                data['size'] =  os.path.getsize(filepath)
            files.append(data)

        # TODO: deal with other links in conf
        for i in conf:

            if i in filenames or i.startswith('/'):
                continue
            if i.startswith('http://') or i.startswith('https://'):
                files.append({'path': i,
                              'name': i,
                              'type': link})

        for f in files:
            f['description'] = conf.get(f['name'], None)

        return files

    def conf(self, path, cascade=None):
        """returns configuration dictionary appropriate to a path"""
        if cascade is None:
            cascase = self.cascade

        directory = os.path.join(self.directory, path.strip('/'))
        if path.strip('/'):
            path_tuple = tuple(path.strip('/').split('/'))
        else:
            path_tuple = ()

        # return cached configuration
        if hasattr(self, '_conf') and path_tuple in self._conf:
            return self._conf[path_tuple]

        conf = OrderedDict()

        # local configuration
        ini_path = os.path.join(directory, 'index.ini')
        if os.path.exists(ini_path):
            _conf = ConfigMunger(ini_path).dict()
            if len(_conf) == 1:
                conf = _conf[_conf.keys()[0]].copy()

        # global configuration
        if not conf and self.configuration and os.path.exists(self.configuration):
            conf = ConfigMunger(self.configuration).dict().get('/%s' % path.rstrip('/'), {})

        # inherit and cascade configuration
        inherit_directory = None
        if '/inherit' in conf:
            inherit_directory = conf['/inherit']
        elif self.cascade and path_tuple:
            inherit_directory = '/%s' % '/'.join(path_tuple[:-1])
        if inherit_directory:
            parent_configuration = self.conf(inherit_directory)
            for key, value in parent_configuration.items():
                if key.startswith('/') and key not in conf:
                    conf[key] = value

        # cache configuration
        if not self.auto_reload:
            if not hasattr(self, '_conf'):
                self._conf = OrderedDict()
            self._conf[path_tuple] = conf

        return conf

    def get_formatters(self, path):
        """return formatters for a path"""
        retval = []
        conf = self.conf(path)
        # apply formatters
        # XXX this should be cached if not self.auto_reload
        if '/formatters' in conf:
            # ordered list of formatters to be applied first
            formatters = [ i for i in conf['/formatters'].split()
                           if i in self.formatters ]
        else:
            formatters = []
        for key in conf:
            if key.startswith('/'):
                key = key[1:]
                if key in self.formatters and key not in formatters:
                    formatters.append(key)
        for name in formatters:
            retval.append(self.formatters[name](conf.get('/%s' % name, '')))
        return retval



#!/usr/bin/env python

"""
serve directory with decoupage, e.g.

``decoupage --port 8080 /home/jhammel/tmp``

If the directory is not specified, the current working directory is used
"""

import optparse
import os
import socket
import sys

from .formatters import Datestamp, Sort, Up, DirectoryIndicator
from .web import Decoupage
from wsgiref import simple_server

here = os.path.dirname(os.path.realpath(__file__))

class DecoupageServer(Decoupage):
    """serve locally with decoupage"""
    # TODO: deprecate; move Decoupage to a few classes
    # with more flexible formatters
    def __init__(self, *args, **kwargs):
        Decoupage.__init__(self, **kwargs)
        # default formatters
        # TODO: make configurable
        self._formatters = [Sort(),
                            DirectoryIndicator('/'),
                            Up('..'),
                            Datestamp('modified: %m %d, %Y')]
    def get_formatters(self, path):
        return self._formatters


def main(args=sys.argv[1:]):

    # parse command line options
    usage = '%prog [options]'
    parser = optparse.OptionParser(usage=usage, description=__doc__)
    parser.add_option('-p', '--port', dest='port',
                      type='int', default=1977,
                      help="port to serve on [DEFAULT: %default]")
    parser.add_option('-a', '--address', dest='address',
                      default='0.0.0.0',
                      help="address to serve on [DEFAULT: %default]")
    parser.add_option('--no-reload', dest='auto_reload',
                      action='store_false', default=True,
                      help="do not dynamically refresh indices")
    parser.add_option('--no-print-ip', dest='print_ip',
                      action='store_false', default=True,
                      help="do not print resolvable IP address")
    options, args = parser.parse_args(args)
    if not args:
        directory = os.getcwd()
    elif len(args) == 1:
        directory = args[0]
    if len(args) > 1:
            # TODO:
            # allow multiple directories with mount points
            #   e.g. `decoupage [options] directory [directory2=/foo] [...]`
            #   This may be done by creating a temporary directory with appropriate
            #   symbolic links (on OSes that allow them)

        parser.print_help()
        parser.exit(1)
    if not os.path.isdir(directory):
        raise OSError("'%s' is not a directory" % directory)

    # create WSGI app
    # TODO:
    # - allow CLI specification of formatters
    # - template specification
    app = DecoupageServer(directory=directory,
                          auto_reload=options.auto_reload)


    # create server
    # TODO: allow choice amongst server classes
    printable_address = '127.0.0.1' if options.address == '0.0.0.0' else options.address
    server = simple_server.make_server(options.address, options.port, app)
    print 'serving directory %s ( %s ) at \nhttp://%s:%d/' % (directory,
                                                              'file://' + directory, # XXX
                                                              printable_address,
                                                              options.port)
    if options.print_ip:
        # from http://stackoverflow.com/questions/166506/finding-local-ip-addresses-using-pythons-stdlib
        hostname = "google.com"
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect((hostname,80))
        hostname = s.getsockname()[0]
        print "http://%s:%s/" % (hostname, options.port)
        s.close()

    # serve directory contents via decoupage
    try:
        server.serve_forever()
    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
functionality related to templates
"""

import argparse
import os
import sys
from .index import index
from pkg_resources import iter_entry_points
from pkg_resources import resource_filename

def template_dirs():
    """registered template directories"""

    template_dirs = set()
    for formatter in iter_entry_points('decoupage.formatters'):
        try:
            formatter.load()
        except:
            continue
        template_dir = resource_filename(formatter.module_name, 'templates')
        if os.path.isdir(template_dir):
            template_dirs.add(template_dir)
    return template_dirs


def templates():
    """templates"""
    templates = []
    for directory in template_dirs():
        templates.extend([os.path.join(directory, filename)
                          for filename in os.listdir(directory)
                          if filename.endswith('.html')])
    return templates

def template_dict():
    """return a dict of templates"""
    return {os.path.basename(template):template for template in templates()}

def main(args=sys.argv[1:]):
    """CLI"""

    # parse command line
    description = 'list and output available templates'
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('template', nargs='?',
                        help="output this template")
    parser.add_argument('-o', '--output', dest='output',
                        help="output to file or directory or stdout")
    # TODO
    parser.add_argument('--cwd', dest='cwd',
                        help="output to current working directory")
    options = parser.parse_args(args)

    # retrieve templates
    _templates = template_dict()

    template = options.template
    if template:

        # look up template
        if not template.endswith('.html'):
            template = template + '.html'
        filename = _templates.get(template)
        if filename is None:
            parser.error("Template '{}' not in {}".format(template, ', '.join(sorted(_templates.keys()))))
        content = open(filename, 'r').read()

        # divine output
        output = options.output
        if output:
            if os.path.isdir(output):
                output = os.path.join(output, 'index.html')
            with open(output, 'w') as f:
                f.write(content)

            directory = os.path.dirname(os.path.abspath(output))
            ini = os.path.join(directory, 'index.ini')
            if not os.path.exists(ini):
                pass
                # TODO: output directory contents to ini
                # if specified

        else:
            print (content)

    else:
        # list templates
        for template in templates():
            print (template)


if __name__ == '__main__':
    main()



"""
pluggable formats for directory listings
"""

import datetime
import PyRSS2Gen
from utils import link
try:
  import json as json
except ImportError:
  import simplejson as json

class JSON(object):
  """
  JSON format for index pages
  just (basically) return the data
  """

  def __init__(self, app):
    self.app = app

  def __call__(self, request, data):

    # fix datetime
    for f in data['files']:
      if 'modified' in f:
        f['modified'] = f['modified'].ctime()

    return 'application/json', json.dumps(data['files'])

class RSS(object):
  """RSS for indices"""

  def __init__(self, app, count=10, cascade=False):
    self.app = app # the decoupage
    self.count = int(count)
    self.cascade = cascade

  def __call__(self, request, data):
    items = [ PyRSS2Gen.RSSItem(title=item['name'],
                                description=item['description'] or item.get('title') or item['name'],
                                pubDate=item['modified'],
                                guid=PyRSS2Gen.Guid(link(request, item['path'])))
              for item in data['files'] ]
    path_link = link(request, data['path'])
    description = data.get('title') or data['path']
    rss = PyRSS2Gen.RSS2(title=description,
                         link=path_link,
                         description=description,
                         lastBuildDate = datetime.datetime.now(),
                         items=items
                         )
    return 'application/rss+xml', rss.to_xml()



from web import Decoupage
from paste.httpexceptions import HTTPExceptionHandler

def namespace_conf(keystr, app_conf):
    keystr += '.'
    return dict([(key.split(keystr, 1)[-1], value)
                 for key, value in app_conf.items()
                 if key.startswith(keystr) ])        


def factory(global_conf, **app_conf):
    """create a webob view and wrap it in middleware"""
    app_conf = namespace_conf('decoupage', app_conf)
    app = Decoupage(**app_conf)
    return HTTPExceptionHandler(app)
    



#!/usr/bin/env python

"""
create index.ini file from directory listings
"""

#ls -1 | while read line; do echo "${line} = ${line}"; done > index.ini

# imports
import argparse
import os
import sys


class CreateIndex(object):
    """
    decoupage directory index .ini creation
    """
    # TODO: maybe this should inherit or otherwise extend
    # some more abstract Index class

    def __init__(self, directory):
        assert os.path.isdir(directory)
        self.directory = directory

    def __str__(self):
        lines = ['{item}={item}'.format(item=item)
                 for item in sorted(os.listdir(self.directory))]
        return '\n'.join(lines) + '\n'


def main(args=sys.argv[1:]):
    """CLI"""

    # parse command line
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('-d', '--directory',
                        default=os.getcwd(),
                        help="directory to create index for (current working directory by default)")
    options = parser.parse_args(args)

    print CreateIndex(options.directory)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
index.ini
"""

import argparse
import os
import subprocess
import sys

here = os.path.dirname(os.path.realpath(__file__))
string = (str, unicode)

def index(directory):
    return '\n'.join(['{name} = {name}'.format(name=name)
                      for name in sorted(os.listdir(directory), key=lambda name: name.lower())
                      if not name.startswith('.')])

def main(args=sys.argv[1:]):

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('directory', help='directory')
    parser.add_argument('-o', '--output', dest='output',
                        type=argparse.FileType('w'), default=sys.stdout,
                        help='output')
    options = parser.parse_args(args)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

import random
import sys
from datetime import datetime
from fnmatch import fnmatch
from pkg_resources import iter_entry_points

### abstract base classes for formatters

class FormatterBase(object):
    """
    abstract base class if you want to use __init__ methods
    in the form of
    'arg1, arg2, arg3, kw1=foo, kw2=bar, kw3=baz
    """

    defaults = {} # default values for attrs to be set on the instance

    def __init__(self, string):
        self._string = string
        args = [ i.strip() for i in string.split(',')]
        for index, arg in enumerate(args):
            if '=' in arg:
                break
        else:
            self.args = args
            for key, default in self.defaults.items():
                setattr(self, key, default)
            return
        self.args = args[:index]
        self.kw = dict([i.split('=', 1) for i in args[index:]])
        for key, default in self.defaults.items():
            value = self.kw.pop(key, default)
            setattr(self, key, value)


### formatters

class Ignore(object):
    """
    ignore files of a glob patterns.
    These files will not be linked to in the template.
    e.g. /ignore = .* *.pdf  # don't list dotfiles and PDFs
    """

    def __init__(self, ignore):
        self.match = ignore.split()

    def __call__(self, request, data):
        _files = []
        for f in data['files']:
            for pattern in self.match:
                if fnmatch(f['name'], pattern):
                    break
            else:
                _files.append(f)
        data['files'] = _files


class All(object):
    """
    only pass files of a certain pattern;
    the inverse of ignore
    calling all with no arguments means only files with descriptions are used
    """

    def __init__(self, pattern):
        self.match = pattern.split()

    def __call__(self, request, data):
        _files = []
        for f in data['files']:
            if self.match:
                for pattern in self.match:
                    if fnmatch(f['name'], pattern):
                        _files.append(f)
                        break
            else:
                # use only files where the title or description is not None
                if (f['description'] is not None) or (f.get('title') is not None):
                    _files.append(f)
        data['files'] = _files


class Sort(FormatterBase):
    """
    determines how to sort the files in a directory;
    right now only by case-insensitive alphabetically
    * reverse : reverse the order of the sorting
    """
    defaults = {'order': 'name'}

    def __init__(self, pattern=''):
        FormatterBase.__init__(self, pattern)
        self.orders = {'name': self.name,
                       'random': self.random,
                       }
        # TODO: date

    def __call__(self, request, data):

        sort = self.orders.get(request.GET.get('order', self.order), self.name)
        data['files'] = sort(data['files'])

        if 'reverse' in self.args:
            data['files'] = list(reversed(data['files']))

    def name(self, files):
        return sorted(files, key=lambda x: x['name'].lower())

    def random(self, files):
        random.shuffle(files)
        return files


class Order(object):
    """
    put the files in a particular order
    """
    def __init__(self, pattern):
        if '=' in pattern:
            key, value = pattern.split('=', 1)
            assert key == 'file'
            self.file = value
        else:
            self.order = [i.strip() for i in pattern.split(',')]

    def __call__(self, request, data):

        if self.file:
            raise NotImplementedError

        files = []
        file_hash = dict([(i['name'], i) for i in data['files']])
        for f in self.order:
            files.append(file_hash.get(f, None))
        files = [ i for i in files if i is not None ]

class DirectoryIndicator(FormatterBase):
    """indicate a directory"""
    indicator = '/'
    def __init__(self, indicator):
        self.indicator = indicator.strip() or self.indicator
    def __call__(self, request, data):
        for f in data['files']:
            if f.get('type') == 'directory':
                title = f.get('title')
                if title is not None:
                    f['title'] = '%s %s' % (title, self.indicator)
                else:
                    description = f.get('description') or f['name']
                    f['description'] = '%s %s' % (description, self.indicator)

class FilenameDescription(FormatterBase):
    """
    obtain the description from the filename
    the file extension (if any) will be dropped and
    spaces will be substituted for underscores
    """

    # TODO : deal with CamelCaseFilenames

    separators = ['_', '-'] # space substitute separators
    lesser_words = [ 'or', 'a', 'the', 'on', 'of' ] # unimportant words

    def __call__(self, request, data):
        for f in data['files']:
            if f['description'] is None:
                description = f['name']
                if '.' in description:
                    description = description.rsplit('.', 1)[0]
                decription = description.strip('_')
                for separator in self.separators:
                    if separator in description:
                        description = ' '.join([(i in self.lesser_words) and i or i.title()
                                                for i in description.split(separator)])
                        description = description[0].upper() + description[1:]
                f['description'] = description


class TitleDescription(FormatterBase):
    """
    splits a description into a title and a description via a separator in
    the description.  The template will now have an additional variable,
    'title', per file
    Arguments:
    * separator: what separator to use (':' by default)
    """
    # XXX what about setting the page title?

    defaults = { 'separator': ':' }

    def __call__(self, request, data):

        # title webpage
        title = self._string
        if ':' in title:
            _title, url = [i.strip() for i in title.split(':', 1)]
            if '://' in url:
                # XXX could also use urlparse
                title = _title
                data['link'] = url
        data['title'] = title

        # title files
        for f in data['files']:
            if f['description'] and self.separator in f['description']:
                title, description = f['description'].split(self.separator, 1)
                title = title.strip()
                description = description.strip()
                if not title:
                    title = f['name']
                f['title'] = title
                f['description'] = description
            else:
                f['title'] = f['description']
                f['description'] = None

class Datestamp(FormatterBase):
    """
    datestamps for modified times
    """
    # TODO:
    # - currently we only do modified; TODO: created/modified/etc
    # - use modified dateutil from bitsyblog;
    # - e.g. javascript for things like e.g. "Yesterday"
    key = 'modified'

    def __init__(self, string):
        FormatterBase.__init__(self, string)

        # check formatting string now v later
        datetime.now().strftime(self._string)

    def __call__(self, request, data):
        for f in data['files']:
            _datetime = f.get(self.key)
            if not _datetime:
                continue
            try:
                datestamp = _datetime.strftime(self._string)
                f['datestamp'] = datestamp
            except:
                raise # TODO: handle more better

class Links(FormatterBase):
    """
    allow list of links per item:
    foo.html = description of foo; [PDF]=foo.pdf; [TXT]=foo.txt
    """

    fatal = False
    defaults = {'separator': ';'}

    def __call__(self, request, data):
        for f in data['files']:
            if f['description'] and self.separator in f['description']:
                description, links = f['description'].split(self.separator, 1)
                links = links.rstrip().split(self.separator)

                if not min(['=' in link for link in links]):
                    message = """%s

%s: misformatted description: %s (separator: %s)""" % (f,
                                                       self.__class__.__name__,
                                                       description,
                                                       self.separator)
                    if self.fatal:
                        raise AssertionError(message)
                    else:
                        print >> sys.stderr, message
                        continue

                f['description'] = description

                links = [link.split('=', 1) for link in links]
                f['links'] = [{'text': text, 'link': link}
                              for text, link in links]


class Up(object):
    """
    provides an up link to the path above:
    /up = ..
    """

    def __init__(self, arg):
        self.up = arg.strip()

    def __call__(self, request, data):
        path = request.path_info
        if (path != '/') and self.up:
            data['files'].insert(0, {'path': '..',
                                     'type': 'directory',
                                     'name': path.rsplit('/', 1)[0] + '/',
                                     'description': self.up})

class CSS(object):
    """specify CSS used (whitespace separated list)"""

    def __init__(self, arg):
        self.css = arg.split()
    def __call__(self, request, data):
        data['css'] = self.css

class JavaScript(object):
    """specify JS for the page"""

    def __init__(self, arg):
        self.scripts = arg.split()
    def __call__(self, request, data):
        data['scripts'] = self.scripts

class Favicon(object):
    """specify favicons for the page"""

    def __init__(self, icon):
        self.icon = icon
    def __call__(self, request, data):
        data['icon'] = self.icon

class Include(object):
    """include a file at the top of the body"""

    def __init__(self, arg):
        self.include = arg
    def __call__(self, request, data):
        data['include'] = self.include


### general purpose functions for formatters

def formatters():
    formatters = {}
    for entry_point in iter_entry_points('decoupage.formatters'):
        try:
            formatter = entry_point.load()
        except:
            continue
        formatters[entry_point.name] = formatter
    return formatters

def main(args=sys.argv[1:]):
    for name, formatter in formatters().items():
        print '%s%s' % (name, formatter.__doc__ and ': ' + formatter.__doc__ or '')

if __name__ == '__main__':
    main()
